/*

The MIT License (MIT)

Copyright (c) 2015 Veit Jahns

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

*/

#include <algorithm>
#include <iostream>
#include <string>
#include <vector>

#include <boost/assign/list_of.hpp>
#include <boost/bind.hpp>

class person
{
public:

	person(const std::string &initName) : name(initName) {}

	const std::string &get_name() const
	{
		return name;
	}

	char get_name_letter(const int index) const
	{
		return name[index];
	}

private:

	std::string name;
};

void example3()
{
	std::cout << "Example 3" << std::endl;

	typedef std::vector<person> person_vector;

	person_vector persons = boost::assign::list_of
		(person("Heinz"))
		(person("Helmut"))
		(person("Siegfried"))
		(person("Hagen"))
		(person("Herbert"));

	person_vector::iterator it =
		std::find_if(
			persons.begin(),
			persons.end(),
			boost::bind(&person::get_name, _1) == "Siegfried");

	if (it != persons.end())
	{
		std::cout << it->get_name() << std::endl;
	}

	std::cout << std::endl;

	it =
		std::find_if(
		persons.begin(),
		persons.end(),
		boost::bind(&person::get_name_letter, _1, 3) == 'm');

	if (it != persons.end())
	{
		std::cout << it->get_name() << std::endl;
	}
}