/*

The MIT License (MIT)

Copyright (c) 2015 Veit Jahns

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

*/

#include <algorithm>
#include <iostream>
#include <string>
#include <vector>

#include <boost/algorithm/string/case_conv.hpp>
#include <boost/assign/list_of.hpp>
#include <boost/bind.hpp>
#include <boost/function.hpp>

typedef boost::function<void(const std::string &)> Sayer;

class person
{
public:

	void say(const std::string &message) const
	{
		std::cout << message << std::endl;
	}
};

class robot
{
public:

	void say(const std::string &message, const bool is_loud) const
	{
		if (is_loud)
		{
			std::cout << boost::to_upper_copy(message) << std::endl;
		}
		else
		{
			std::cout << boost::to_lower_copy(message) << std::endl;
		}
	}
};

void greet(Sayer greeter)
{
	greeter(std::string("Welcome to our hotel!"));
}

void produceGreeting(const std::string &message)
{
	std::cout << message << std::endl;
}

void example4()
{
	std::cout << "Example 4" << std::endl;

	person p;
	robot r;

	Sayer s1 = boost::bind(&person::say, &p, _1);
	Sayer s2 = boost::bind(&robot::say, &r, _1, true);
	Sayer s3 = boost::bind(produceGreeting, _1);

	greet(s1);
	greet(s2);
	greet(s3);
}